@@ -41,7 +41,7 @@ compat_get_temp_path( void )
 }
 
 const char*
-compat_get_home_path( void )
+compat_get_config_path( void )
 {
   return "PROGDIR:settings";
 }
