@@ -158,7 +158,7 @@ startup_manager_run( void )
 
   /* If there are still any modules left to be called, then that's bad */
   if( registered_modules->len ) {
-    ui_error( UI_ERROR_ERROR, "%d startup modules could not be called",
+    ui_error( UI_ERROR_ERROR, "%u startup modules could not be called",
               registered_modules->len );
     return 1;
   }
