@@ -1,8 +1,6 @@
 /* spec_se.c: ZX Spectrum SE specific routines
    Copyright (c) 1999-2015 Fredrick Meunier, Philip Kendall, Darren Salt
 
-   $Id: spec_se.c 5434 2016-05-01 04:22:45Z fredm $
-
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -35,7 +33,7 @@
 #include "machine.h"
 #include "machines.h"
 #include "machines_periph.h"
-#include "memory.h"
+#include "memory_pages.h"
 #include "peripherals/dck.h"
 #include "peripherals/scld.h"
 #include "snapshot.h"
