@@ -725,7 +725,9 @@ no_rs232_in:
 
   if( if1_ula.s_net_mode == 0 ) {		/* if we do raw */
     /* Here is the input routine */
-    read( if1_ula.fd_net, &if1_ula.net, 1 );	/* Ok, if no byte, we send last*/
+    ssize_t unused;
+    unused = read( if1_ula.fd_net, &if1_ula.net, 1 );	/* Ok, if no byte, we send last*/
+    (void) unused;
   } else {/* if( if1_ula.s_net_mode == 1 ) if we do interpreted */
 /* Here is the input routine. There are several stage in input
    and output. So first for output. if1 first do SEND-SC 
