@@ -2,8 +2,6 @@
    Copyright (c) 2002-2015 Alexander Yurchenko, Russell Marks, Philip Kendall,
 			   Fredrick Meunier
 
-   $Id: sdlsound.c 5434 2016-05-01 04:22:45Z fredm $
-
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -95,7 +93,11 @@ sound_lowlevel_init( const char *device, int *freqptr, int *stereoptr )
      speed to about 2000% on my Mac, 100Hz allows up to 5000% for me) */
   if( hz > 100.0 ) hz = 100.0;
   sound_framesiz = *freqptr / hz;
+#ifdef __FreeBSD__
+  requested.samples = pow( 2.0, floor( log2( sound_framesiz ) ) );
+#else			/* #ifdef __FreeBSD__ */
   requested.samples = sound_framesiz;
+#endif			/* #ifdef __FreeBSD__ */
 
   if ( SDL_OpenAudio( &requested, &received ) < 0 ) {
     settings_current.sound = 0;
