@@ -28,7 +28,7 @@
 
 #include <libspectrum.h>
 
-#include "memory.h"
+#include "memory_pages.h"
 
 /* How many tstates have elapsed since the last interrupt? (or more
    precisely, since the ULA last pulled the /INT line to the Z80 low) */
