@@ -1,4 +1,4 @@
-/* gtkmouse.c: GTK+ routines for emulating Spectrum mice
+/* gtkmouse.c: GTK routines for emulating Spectrum mice
    Copyright (c) 2004 Darren Salt
    Copyright (c) 2015 Sergio Baldoví
 
@@ -34,15 +34,76 @@
 #include <gdk/gdkwayland.h>
 #endif
 
+#ifdef GDK_WINDOWING_X11
 /* For XWarpPointer *only* - see below */
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
+#endif
 
 static GdkCursor *nullpointer = NULL;
 
 /* The widget we base our events, grabs, warping etc on */
 static GtkWidget *mouse_widget = NULL;
 
+/* Translate absolute pointer coordinate to relative movement */
+static void (*mouse_motion_fn)( gdouble x, gdouble y, int *rel_x, int *rel_y );
+
+#if defined GDK_WINDOWING_WAYLAND || defined GDK_WINDOWING_WIN32
+
+/* On Wayland we can't warp the pointer so we keep the last position */
+static gdouble last_pos_x = 0;
+static gdouble last_pos_y = 0;
+static int have_last_position = 0;
+
+static void
+mouse_motion_relative( gdouble x, gdouble y, int *rel_x, int *rel_y )
+{
+   if( have_last_position ) {
+     *rel_x = x - last_pos_x;
+     *rel_y = y - last_pos_y;
+   } else {
+     *rel_x = 0;
+     *rel_y = 0;
+     have_last_position = 1;
+   }
+
+   last_pos_x = x;
+   last_pos_y = y;
+}
+
+#endif                /* if defined GDK_WINDOWING_WAYLAND ||
+                            defined GDK_WINDOWING_WIN32 */
+
+#ifdef GDK_WINDOWING_X11
+
+static void
+mouse_motion_x11( gdouble x, gdouble y, int *rel_x, int *rel_y )
+{
+  *rel_x = x - 128;
+  *rel_y = y - 128;
+
+  if( x != 128 || y != 128 ) {
+    GdkWindow *window = gtk_widget_get_window( mouse_widget );
+    XWarpPointer( GDK_WINDOW_XDISPLAY( window ), None,
+                  GDK_WINDOW_XID( window ), 0, 0, 0, 0, 128, 128 );
+  }
+}
+
+#endif                /* #ifdef GDK_WINDOWING_X11 */
+
+#ifdef GDK_WINDOWING_WIN32
+
+static void
+mouse_motion_win32( gdouble x, gdouble y, int *rel_x, int *rel_y )
+{
+  mouse_motion_relative( x, y, rel_x, rel_y );
+
+  /* Keep pointer hidden */
+  SetCursor( NULL );
+}
+
+#endif                /* #ifdef GDK_WINDOWING_WIN32 */
+
 static void
 gtkmouse_reset_pointer( void )
 {
@@ -50,34 +111,59 @@ gtkmouse_reset_pointer( void )
 
      The logic here is a bit hairy:
 
-     * On GTK+ 2.x, we warp relative to the drawing area
-     * On GTK+ 3.x on X11, we warp relative to the top-level window
-     * On GTK+ 3.x on Wayland, we don't warp at all because it causes a
+     * On GTK 2.x, we warp relative to the drawing area
+     * On GTK 3.x on X11, we warp relative to the top-level window
+     * On GTK 3.x on Wayland, we don't warp at all because it causes a
        segfault (see bug #435)
+     * On GTK 3.x on win32, we don't warp at all
    */
 
-  GdkWindow *window;
-
 #ifdef GDK_WINDOWING_WAYLAND
+
   GdkDisplay *display = gdk_display_get_default();
-  if( GDK_IS_WAYLAND_DISPLAY( display ) ) return;
+  if( GDK_IS_WAYLAND_DISPLAY( display ) ) {
+    mouse_motion_fn = mouse_motion_relative;
+    have_last_position = 0;
+    return;
+  }
+
 #endif                /* #ifdef GDK_WINDOWING_WAYLAND */
 
-  window = gtk_widget_get_window( mouse_widget );
+#ifdef GDK_WINDOWING_X11
+
+  mouse_motion_fn = mouse_motion_x11;
+
+  /* Force initial position */
+  GdkWindow *window = gtk_widget_get_window( mouse_widget );
   XWarpPointer( GDK_WINDOW_XDISPLAY( window ), None, 
                 GDK_WINDOW_XID( window ), 0, 0, 0, 0, 128, 128 );
+  return;
+
+#endif                /* #ifdef GDK_WINDOWING_WAYLAND */
+
+#ifdef GDK_WINDOWING_WIN32
+
+  mouse_motion_fn = mouse_motion_win32;
+  have_last_position = 0;
+  return;
+
+#endif                /* #ifdef GDK_WINDOWING_WIN32 */
+
 }
 
 static gboolean
 motion_event( GtkWidget *widget GCC_UNUSED, GdkEventMotion *event,
               gpointer data GCC_UNUSED )
 {
-  if( !ui_mouse_grabbed ) return TRUE;
+  int rel_x, rel_y;
 
-  if( event->x != 128 || event->y != 128 )
-    gtkmouse_reset_pointer();
-  ui_mouse_motion( event->x - 128, event->y - 128 );
-  return TRUE;
+  if( !ui_mouse_grabbed ) return FALSE;
+
+  /* Get relative movement from last position */
+  (*mouse_motion_fn)( event->x, event->y, &rel_x, &rel_y );
+  ui_mouse_motion( rel_x, rel_y );
+
+  return FALSE;
 }
 
 static gboolean
@@ -89,7 +175,9 @@ button_event( GtkWidget *widget GCC_UNUSED, GdkEventButton *event,
     ui_mouse_button( event->button, 1 );
   else
     ui_mouse_button( event->button, 0 );
-  return TRUE;
+
+  /* Stop other handlers only if we've grabbed the mouse */
+  return ui_mouse_grabbed? TRUE : FALSE;
 }
 
 void
@@ -121,7 +209,7 @@ ui_mouse_grab( int startup )
 
   window = gtk_widget_get_window( mouse_widget );
 
-#if !GTK_CHECK_VERSION( 3, 0, 0 )
+#if !GTK_CHECK_VERSION( 3, 20, 0 )
 
   if( !nullpointer ) {
     nullpointer = gdk_cursor_new( GDK_BLANK_CURSOR );
@@ -147,7 +235,7 @@ ui_mouse_grab( int startup )
   status = gdk_seat_grab( seat, window, GDK_SEAT_CAPABILITY_ALL_POINTING,
                           FALSE, nullpointer, NULL, NULL, NULL );
 
-#endif                /* #if !GTK_CHECK_VERSION( 3, 0, 0 ) */
+#endif                /* #if !GTK_CHECK_VERSION( 3, 20, 0 ) */
 
   if( status == GDK_GRAB_SUCCESS ) {
     gtkmouse_reset_pointer();
@@ -162,7 +250,7 @@ ui_mouse_grab( int startup )
 int
 ui_mouse_release( int suspend GCC_UNUSED )
 {
-#if !GTK_CHECK_VERSION( 3, 0, 0 )
+#if !GTK_CHECK_VERSION( 3, 20, 0 )
 
   gdk_pointer_ungrab( GDK_CURRENT_TIME );
 
@@ -175,7 +263,7 @@ ui_mouse_release( int suspend GCC_UNUSED )
   seat = gdk_display_get_default_seat( display );
   gdk_seat_ungrab( seat );
 
-#endif                /* #if !GTK_CHECK_VERSION( 3, 0, 0 ) */ 
+#endif                /* #if !GTK_CHECK_VERSION( 3, 20, 0 ) */
 
   ui_statusbar_update( UI_STATUSBAR_ITEM_MOUSE, UI_STATUSBAR_STATE_INACTIVE );
   return 0;
