@@ -361,6 +361,26 @@ widget_menu_keyhandler( input_key key )
     }
     break;
 
+  case INPUT_KEY_Home:
+    new_highlight_line = 0;
+    ptr = &menu[1 + new_highlight_line];
+    while( new_highlight_line < (ptrdiff_t)count - 1 && ptr->inactive ) {
+      new_highlight_line += 1;
+      ptr = &menu[1 + new_highlight_line];
+    }
+    cursor_pressed = 1;
+    break;
+
+  case INPUT_KEY_End:
+    new_highlight_line = (ptrdiff_t)count - 1;
+    ptr = &menu[1 + new_highlight_line];
+    while( new_highlight_line > 0 && ptr->inactive ) {
+      new_highlight_line -= 1;
+      ptr = &menu[1 + new_highlight_line];
+    }
+    cursor_pressed = 1;
+    break;
+
   default:	/* Keep gcc happy */
     break;
 
