@@ -56,6 +56,9 @@ int icons_part_width = 140; /* will be calculated dynamically later */
 int icons_part_height = 27;
 int icons_part_margin = 2;
 
+/* Status bar handle */
+HWND fuse_hStatusWindow;
+
 void
 win32statusbar_create( HWND hWnd )
 {
