@@ -1,8 +1,6 @@
 /* z80_internals.h: routines which are internal to the Z80 core
    Copyright (c) 2016 Philip Kendall
 
-   $Id$
-
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -26,6 +24,6 @@
 #ifndef FUSE_Z80_INTERNALS_H
 #define FUSE_Z80_INTERNALS_H
 
-void z80_debugger_variables_init();
+void z80_debugger_variables_init( void );
 
 #endif			/* #ifndef FUSE_Z80_INTERNALS */
